@@ -2,11 +2,6 @@ import json
 from io import StringIO
 from unittest.mock import (patch, mock_open, MagicMock)
 
-from linkmanager.settings import INDENT
-from linkmanager import interface
-tty_i = interface()
-tty_i.test = True
-
 from linkmanager.translation import gettext as _
 
 
@@ -25,50 +20,58 @@ addlink = iter([
 
     ### input on: test_cmd_addlinks
     'http://link1.com http://link2.com http://link3.com',
-    'link1_tag1, link1_tag2, link1_tag3',
-    '',
+    'link1_tag1 link1_tag2 link1_tag3',
+    1,
     'link_1 description...',
+    '',
 
-    'link2_tag1, link2_tag2',
+    'link2_tag1 link2_tag2',
     5,
     'link_2 description...',
+    'link2_title',
 
     'link3_tag1',
     'incorrect priority value',
     15,
     5,
     'link_3 description...',
+    'link3_title',
 
     ### input on: test_cmd_addlinks_with_update
     'http://link2.com http://link3.com http://link4.com',
     _('n'),
     # like Yes
     '',
-    'link3_tag1, link3_tag2, link3_tag3',
+    'link3_tag1 link3_tag2 link3_tag3',
     7,
     'link_3 description...',
+    '',
 
     'link4_tag1',
     8,
     'link_4 description...',
+    '',
 
     ### input on: test_cmd_updatelinks
     'http://link1.com',
-    'link1_tag1, link1_tag3, link1_tag4',
+    'link1_tag1 link1_tag3 link1_tag4',
     2,
     'link_1 description...',
+    '',
 
     ### input on: test_cmd_updatelinks_with_add
     'http://link3.com http://link5.com http://link6.com',
-    'link3_tag1, link3_tag2, link3_tag3',
+    'link3_tag1 link3_tag2 link3_tag3',
     10,
     'link_3 new description...',
+    '',
 
     _('n'),
     _('Y'),
-    'link6_tag1, link6_tag2, link6_tag3',
+    'link6_tag1 link6_tag2 link6_tag3',
     9,
     'link_6 description...',
+    '',
 
     ### input on: test_cmd_removelinks
     ''
@@ -76,17 +79,28 @@ addlink = iter([
 
 
 def get_input(string):
-    print(string)
     return next(addlink)
 
 
+tty_i = None
+from linkmanager.tests import fakesettings
+import linkmanager
+fakesettings.INDENT = 4
+fakesettings.ACTIVE_CACHE = False
+linkmanager.settings = fakesettings
+INDENT = fakesettings.INDENT
+
+
 @patch('builtins.input', get_input)
 @patch('sys.stdout', new_callable=StringIO)
 def test_cmd_flush(mock_stdout):
+    from linkmanager import interface
+    global tty_i
+    tty_i = interface(test=True)
     assert tty_i.flush() is True
     assert mock_stdout.getvalue() == ''.join([
         _("You're about to empty the entire Database."),
-        _("Are you sure [Y/n] ?") + " \n",
+        _("Are you sure [Y/n] ?"),
         _("Database entirely flushed.") + "\n"
     ])
     mock_stdout.truncate(0)
@@ -94,81 +108,106 @@ def test_cmd_flush(mock_stdout):
     assert tty_i.flush() is False
     assert mock_stdout.getvalue() == ''.join([
         _("You're about to empty the entire Database."),
-        _("Are you sure [Y/n] ?") + " \n"
+        _("Are you sure [Y/n] ?")
     ])
 
 
+class FakeURL:
+    def result(self):
+        class Result:
+            url = 'http://fakeurl.com'
+            content = '<html><head><title>Fake Title</title></head></html>'
+            status_code = 200
+        return Result()
+
+
+def session_get(link, url):
+    return FakeURL()
+
+
+@patch('requests_futures.sessions.FuturesSession.get', session_get)
 @patch('builtins.input', get_input)
 @patch('sys.stdout', new_callable=StringIO)
 @patch('arrow.now', lambda: "2014-02-10T19:59:34.612714+01:00")
 def test_cmd_addlinks(mock_stdout):
     tty_i.flush(forced=['forced'])
     assert mock_stdout.getvalue() == _('Database entirely flushed.') + '\n'
-
     mock_stdout.seek(0)
     assert tty_i.addlinks() is True
-
     assert mock_stdout.getvalue() == ''.join([
-        _('Give one or several links (separate with space)'), ' : \n',
+        _('Give one or several links (separate with spaces)'), ' :',
 
-        _('%s properties') % 'http://link1.com', ' : \n',
-        ' ' * INDENT, _('tags (separate with ",")'), ' : \n',
+        _('%s properties') % 'http://link1.com', ' :\n',
+        ' ' * INDENT, _('tags (at least one, several separate with spaces)'),
+        ' :',
         ' ' * INDENT, _('priority value (integer value between 1 and 10)'),
-        ' : \n',
-        ' ' * INDENT, _('give a description'), ' : \n',
+        ' :',
+        ' ' * INDENT, _('give a description'), ' :',
+        ' ' * INDENT, _('give a title'), ' :',
 
-        _('%s properties') % 'http://link2.com', ' : \n',
-        ' ' * INDENT, _('tags (separate with ",")'), ' : \n',
+        _('%s properties') % 'http://link2.com', ' :\n',
+        ' ' * INDENT, _('tags (at least one, several separate with spaces)'),
+        ' :',
         ' ' * INDENT, _('priority value (integer value between 1 and 10)'),
-        ' : \n',
-        ' ' * INDENT, _('give a description'), ' : \n',
+        ' :',
+        ' ' * INDENT, _('give a description'), ' :',
+        ' ' * INDENT, _('give a title'), ' :',
 
-        _('%s properties') % 'http://link3.com', ' : \n',
-        ' ' * INDENT, _('tags (separate with ",")'), ' : \n',
+        _('%s properties') % 'http://link3.com', ' :\n',
+        ' ' * INDENT, _('tags (at least one, several separate with spaces)'),
+        ' :',
         ' ' * INDENT, _('priority value (integer value between 1 and 10)'),
-        ' : \n',
+        ' :',
         ' ' * INDENT, _('priority value not range between 1 and 10, retry'),
-        ' : \n',
+        ' :',
         ' ' * INDENT, _('priority value not range between 1 and 10, retry'),
-        ' : \n',
-        ' ' * INDENT, _('give a description'), ' : \n'
+        ' :',
+        ' ' * INDENT, _('give a description'), ' :',
+        ' ' * INDENT, _('give a title'), ' :',
     ])
 
 
+@patch('requests_futures.sessions.FuturesSession.get', session_get)
 @patch('builtins.input', get_input)
 @patch('sys.stdout', new_callable=StringIO)
 @patch('arrow.now', lambda: "2014-02-14T10:22:34.612714+01:00")
+@patch('linkmanager.settings.AUTHOR', 'Author name')
 def test_cmd_addlinks_with_update(mock_stdout):
     assert tty_i.addlinks() is True
 
     assert mock_stdout.getvalue() == ''.join([
-        _('Give one or several links (separate with space)'), ' : \n',
+        _('Give one or several links (separate with spaces)'), ' :',
 
         ' ' * INDENT, _(
             'the link "%s" already exist: '
             'do you want to update [Y/n] ?'
-        ) % 'http://link2.com', ' : \n',
+        ) % 'http://link2.com', ' :\n',
 
         ' ' * INDENT, _(
             'the link "%s" already exist: '
             'do you want to update [Y/n] ?'
-        ) % 'http://link3.com', ' : \n',
-        _('%s properties') % 'http://link3.com', ' : \n',
-        ' ' * INDENT, _('tags (separate with ",")'), ' : \n',
+        ) % 'http://link3.com', ' :\n',
+        _('%s properties') % 'http://link3.com', ' :\n',
+        ' ' * INDENT, _('tags (at least one, several separate with spaces)'),
+        ' :',
         ' ' * INDENT, _('priority value (integer value between 1 and 10)'),
-        ' : \n',
-        ' ' * INDENT, _('give a description'), ' : \n',
+        ' :',
+        ' ' * INDENT, _('give a description'), ' :',
+        ' ' * INDENT, _('give a title'), ' :',
 
-        _('%s properties') % 'http://link4.com', ' : \n',
-        ' ' * INDENT, _('tags (separate with ",")'), ' : \n',
+        _('%s properties') % 'http://link4.com', ' :\n',
+        ' ' * INDENT, _('tags (at least one, several separate with spaces)'),
+        ' :',
         ' ' * INDENT, _('priority value (integer value between 1 and 10)'),
-        ' : \n',
-        ' ' * INDENT, _('give a description'), ' : \n',
+        ' :',
+        ' ' * INDENT, _('give a description'), ' :',
+        ' ' * INDENT, _('give a title'), ' :',
     ])
 
 
 dump_afteradd = """{
     "http://link1.com": {
+        "author": "Author name",
         "description": "link_1 description...",
         "init date": "2014-02-10T19:59:34.612714+01:00",
         "priority": "1",
@@ -176,10 +215,10 @@ dump_afteradd = """{
             "link1_tag1",
             "link1_tag2",
             "link1_tag3"
-        ],
-        "update date": "None"
+        ]
     },
     "http://link2.com": {
+        "author": "Author name",
         "description": "link_2 description...",
         "init date": "2014-02-10T19:59:34.612714+01:00",
         "priority": "5",
@@ -187,9 +226,10 @@ dump_afteradd = """{
             "link2_tag1",
             "link2_tag2"
         ],
-        "update date": "None"
+        "title": "link2_title"
     },
     "http://link3.com": {
+        "author": "Author name",
         "description": "link_3 description...",
         "init date": "2014-02-10T19:59:34.612714+01:00",
         "priority": "7",
@@ -201,11 +241,11 @@ dump_afteradd = """{
         "update date": "2014-02-14T10:22:34.612714+01:00"
     },
     "http://link4.com": {
+        "author": "Author name",
         "description": "link_4 description...",
         "init date": "2014-02-14T10:22:34.612714+01:00",
         "priority": "8",
-        "tags": ["link4_tag1"],
-        "update date": "None"
+        "tags": ["link4_tag1"]
     }
 }
 """
@@ -217,38 +257,45 @@ def test_cmd_addlinks_dump(mock_stdout):
     assert json.loads(mock_stdout.getvalue()) == json.loads(dump_afteradd)
 
 
+@patch('requests_futures.sessions.FuturesSession.get', session_get)
 @patch('builtins.input', get_input)
 @patch('sys.stdout', new_callable=StringIO)
 @patch('arrow.now', lambda: "2014-02-15T12:20:34.612714+01:00")
+@patch('linkmanager.settings.AUTHOR', 'Author name')
 def test_cmd_updatelinks(mock_stdout):
     assert tty_i.updatelinks() is True
 
     assert mock_stdout.getvalue() == ''.join([
-        _('Give one or several links (separate with space)'), ' : \n',
+        _('Give one or several links (separate with spaces)'), ' :',
 
-        _('%s properties') % 'http://link1.com', ' : \n',
-        ' ' * INDENT, _('tags (separate with ",")'), ' : \n',
+        _('%s properties') % 'http://link1.com', ' :\n',
+        ' ' * INDENT, _('tags (at least one, several separate with spaces)'),
+        ' :',
         ' ' * INDENT, _('priority value (integer value between 1 and 10)'),
-        ' : \n',
-        ' ' * INDENT, _('give a description'), ' : \n'
+        ' :',
+        ' ' * INDENT, _('give a description'), ' :',
+        ' ' * INDENT, _('give a title'), ' :',
     ])
 
 
+@patch('requests_futures.sessions.FuturesSession.get', session_get)
 @patch('builtins.input', get_input)
 @patch('sys.stdout', new_callable=StringIO)
 @patch('arrow.now', lambda: "2014-02-10T19:59:34.612714+01:00")
+@patch('linkmanager.settings.AUTHOR', 'Author name')
 def test_cmd_updatelinks_with_add(mock_stdout):
     assert tty_i.updatelinks() is True
-    #cp.cpvar(mock_stdout.getvalue())
 
     assert mock_stdout.getvalue() == ''.join([
-        _('Give one or several links (separate with space)'), ' : \n',
+        _('Give one or several links (separate with spaces)'), ' :',
 
-        _('%s properties') % 'http://link3.com', ' : \n',
-        ' ' * INDENT, _('tags (separate with ",")'), ' : \n',
+        _('%s properties') % 'http://link3.com', ' :\n',
+        ' ' * INDENT, _('tags (at least one, several separate with spaces)'),
+        ' :',
         ' ' * INDENT, _('priority value (integer value between 1 and 10)'),
-        ' : \n',
-        ' ' * INDENT, _('give a description'), ' : \n',
+        ' :',
+        ' ' * INDENT, _('give a description'), ' :',
+        ' ' * INDENT, _('give a title'), ' :',
 
         ' ' * INDENT, _(
             'the link "%s" does not exist: '
@@ -260,16 +307,19 @@ def test_cmd_updatelinks_with_add(mock_stdout):
             'do you want to create [Y/n] ?'
         ) % 'http://link6.com', ' : \n',
 
-        _('%s properties') % 'http://link6.com', ' : \n',
-        ' ' * INDENT, _('tags (separate with ",")'), ' : \n',
+        _('%s properties') % 'http://link6.com', ' :\n',
+        ' ' * INDENT, _('tags (at least one, several separate with spaces)'),
+        ' :',
         ' ' * INDENT, _('priority value (integer value between 1 and 10)'),
-        ' : \n',
-        ' ' * INDENT, _('give a description'), ' : \n'
+        ' :',
+        ' ' * INDENT, _('give a description'), ' :',
+        ' ' * INDENT, _('give a title'), ' :'
     ])
 
 
 dump_afterupdate = """{
     "http://link1.com": {
+        "author": "Author name",
         "description": "link_1 description...",
         "init date": "2014-02-10T19:59:34.612714+01:00",
         "priority": "2",
@@ -281,6 +331,7 @@ dump_afterupdate = """{
         "update date": "2014-02-15T12:20:34.612714+01:00"
     },
     "http://link2.com": {
+        "author": "Author name",
         "description": "link_2 description...",
         "init date": "2014-02-10T19:59:34.612714+01:00",
         "priority": "5",
@@ -288,9 +339,10 @@ dump_afterupdate = """{
             "link2_tag1",
             "link2_tag2"
         ],
-        "update date": "None"
+        "title": "link2_title"
     },
     "http://link3.com": {
+        "author": "Author name",
         "description": "link_3 new description...",
         "init date": "2014-02-10T19:59:34.612714+01:00",
         "priority": "10",
@@ -302,15 +354,16 @@ dump_afterupdate = """{
         "update date": "2014-02-10T19:59:34.612714+01:00"
     },
     "http://link4.com": {
+        "author": "Author name",
         "description": "link_4 description...",
         "init date": "2014-02-14T10:22:34.612714+01:00",
         "priority": "8",
         "tags": [
             "link4_tag1"
-        ],
-        "update date": "None"
+        ]
     },
     "http://link6.com": {
+        "author": "Author name",
         "description": "link_6 description...",
         "init date": "2014-02-10T19:59:34.612714+01:00",
         "priority": "9",
@@ -318,8 +371,7 @@ dump_afterupdate = """{
             "link6_tag1",
             "link6_tag2",
             "link6_tag3"
-        ],
-        "update date": "None"
+        ]
     }
 }
 """
@@ -327,10 +379,6 @@ dump_afterupdate = """{
 
 @patch('sys.stdout', new_callable=StringIO)
 def test_cmd_updatelinks_dump(mock_stdout):
-# def test_cmd_updatelinks_dump():
-    # with open('tt.json', 'w') as f:
-    #     f.write(cp.result)
-    # assert False is True
     assert tty_i.dump() is True
     assert json.loads(mock_stdout.getvalue()) == json.loads(dump_afterupdate)
 
@@ -345,7 +393,7 @@ def test_cmd_removelinks(mock_stdout):
         "http://link7.com"
     ]) is True
     assert mock_stdout.getvalue() == ''.join([
-        _('Give one or several links (separate with space)'), ' : \n',
+        _('Give one or several links (separate with spaces)'), ' :',
         _('the link "%s" does not exist.') % "http://link5.com" + '\n',
         _('the link "%s" has been deleted.') % "http://link6.com" + '\n',
         _('the link "%s" does not exist.') % "http://link7.com" + '\n'
@@ -354,6 +402,7 @@ def test_cmd_removelinks(mock_stdout):
 
 dump_afterremove = """{
     "http://link1.com": {
+        "author": "Author name",
         "description": "link_1 description...",
         "init date": "2014-02-10T19:59:34.612714+01:00",
         "priority": "2",
@@ -365,6 +414,7 @@ dump_afterremove = """{
         "update date": "2014-02-15T12:20:34.612714+01:00"
     },
     "http://link2.com": {
+        "author": "Author name",
         "description": "link_2 description...",
         "init date": "2014-02-10T19:59:34.612714+01:00",
         "priority": "5",
@@ -372,9 +422,10 @@ dump_afterremove = """{
             "link2_tag1",
             "link2_tag2"
         ],
-        "update date": "None"
+        "title": "link2_title"
     },
     "http://link3.com": {
+        "author": "Author name",
         "description": "link_3 new description...",
         "init date": "2014-02-10T19:59:34.612714+01:00",
         "priority": "10",
@@ -386,13 +437,13 @@ dump_afterremove = """{
         "update date": "2014-02-10T19:59:34.612714+01:00"
     },
     "http://link4.com": {
+        "author": "Author name",
         "description": "link_4 description...",
         "init date": "2014-02-14T10:22:34.612714+01:00",
         "priority": "8",
         "tags": [
             "link4_tag1"
-        ],
-        "update date": "None"
+        ]
     }
 }
 """
@@ -455,7 +506,8 @@ first_fixture = """{
 
 
 @patch('builtins.open', mock_open(read_data=first_fixture))
-def test_cmd_one_load():
+@patch('sys.stdout', new_callable=StringIO)
+def test_cmd_one_load(mock_stdout):
     tty_i.flush(forced=['forced'])
     # One file
     assert tty_i.load(['file.json']) is True
@@ -475,8 +527,7 @@ second_fixture = """{
         "tags": [
             "benchmark",
             "linux"
-        ],
-        "update date": "None"
+        ]
     }
 }
 """
@@ -491,8 +542,7 @@ third_fixture = """{
             "python",
             "shell",
             "ubuntu"
-        ],
-        "update date": "None"
+        ]
     }
 }
 """
@@ -518,8 +568,7 @@ fourth_fixture = """{
             "joke",
             "linux",
             "math"
-        ],
-        "update date": "None"
+        ]
     }
 }
 """
@@ -543,8 +592,7 @@ fifth_fixture = """{
         "tags": [
             "benchmark",
             "linux"
-        ],
-        "update date": "None"
+        ]
     },
     "http://ubuntu.com": {
         "description": "Fr Ubuntu site",
@@ -555,8 +603,7 @@ fifth_fixture = """{
             "python",
             "shell",
             "ubuntu"
-        ],
-        "update date": "None"
+        ]
     },
     "http://xkcd.com": {
         "description": "A webcomic of romance ...",
@@ -567,8 +614,7 @@ fifth_fixture = """{
             "joke",
             "linux",
             "math"
-        ],
-        "update date": "None"
+        ]
     }
 }
 """
@@ -597,7 +643,8 @@ def multi_mock_open(mock=None, read_data=''):
 
 
 @patch('builtins.open', multi_mock_open())
-def test_cmd_multi_load():
+@patch('sys.stdout', new_callable=StringIO)
+def test_cmd_multi_load(mock_stdout):
     tty_i.flush(forced=['forced'])
     # Several files
     assert tty_i.load(json_files=[
@@ -607,10 +654,6 @@ def test_cmd_multi_load():
 
 @patch('sys.stdout', new_callable=StringIO)
 def test_cmd_dump_after_multi_load(mock_stdout):
-# def test_cmd_dump_after_multi_load():
-    # with open('tt.json', 'w') as f:
-    #     f.write(cp.result)
-    # assert False is True
     assert tty_i.dump() is True
     assert json.loads(mock_stdout.getvalue()) == json.loads(fifth_fixture)
 
@@ -621,10 +664,10 @@ def test_cmd_searchlinks_allresult(mock_stdout):
     assert tty_i.searchlinks() is True
     assert mock_stdout.getvalue() == ''.join([
         _('%s links totally founded') % '4', ' : \n',
-        ' ' * INDENT, 'http://ubuntu.com\n',
-        ' ' * INDENT, 'http://linuxfr.org\n',
-        ' ' * INDENT, 'http://phoronix.com\n',
-        ' ' * INDENT, 'http://xkcd.com\n'
+        ' ' * INDENT, '  1 ➤ http://ubuntu.com\n',
+        ' ' * INDENT, '  2 ➤ http://linuxfr.org\n',
+        ' ' * INDENT, '  3 ➤ http://phoronix.com\n',
+        ' ' * INDENT, '  4 ➤ http://xkcd.com\n'
     ])
 
 
@@ -639,6 +682,13 @@ def test_cmd_searchlinks(mock_stdout):
     assert tty_i.searchlinks(['bsd']) is True
     assert mock_stdout.getvalue() == ''.join([
         _('%s links founded') % '2', ' : \n',
-        ' ' * INDENT, 'http://linuxfr.org\n',
-        ' ' * INDENT, 'http://xkcd.com\n'
+        ' ' * INDENT, '  1 ➤ http://linuxfr.org\n',
+        ' ' * INDENT, '  2 ➤ http://xkcd.com\n'
     ])
+
+# def test_print_dump():
+#     print('\n')
+#     tty_i.dump()
+
+    # with open('fake.json', 'w') as f:
+    #     f.write(mock_stdout.getvalue())
